.installed-overlay-box {
	text-shadow: none;
}

.category-tile:not(.category-tile-iconless) {
	box-shadow: inset 0 0 0 1px alpha(@card_fg_color, .5);
}

.application-details-infobar.info {
	background-color: #d3d7cf;
	border-color: darker(#d3d7cf);
}

.application-details-infobar.warning {
	color: @theme_fg_color;
}

review-bar,
.review-histogram star-image {
	color: alpha(@theme_fg_color, .8);
}

star-image {
	color: inherit;
}

.context-tile-lozenge {
	box-shadow: inset 0 0 0 2px currentColor;
}
